import PropTypes from 'prop-types';
import { useMemo } from 'react';
import classnames from 'classnames';
import classes from './styles.module.scss';

/**
 * A curved arrow that can be used to call out specific elements of a UI.
 */
const ArcArrow = function ArcArrow(props) {
  const {
    start,
    length,
    reversed,
    color,
    classes: customClasses,
  } = props;

  const w = 100;
  const r = w / 2;
  const arrow = 16;

  const arrowProps = useMemo(() => {
    const startRadians = start * (Math.PI / 180);
    const endRadians = (start + length) * (Math.PI / 180);

    const flag = length > 180 ? '0 1 1' : '0 0 1';

    const startX = r + r * Math.cos(startRadians);
    const startY = r + r * Math.sin(startRadians);

    const endX = r + r * Math.cos(endRadians);
    const endY = r + r * Math.sin(endRadians);

    const arrowPos = reversed
      ? { x: startX, y: startY }
      : { x: endX, y: endY };

    const arrowDeg = reversed
      ? start + 180
      : start + length;

    return {
      startX, startY, endX, endY, arrowPos, arrowDeg, flag,
    };
  }, [start, length, reversed]);

  const {
    startX, startY, endX, endY, arrowPos, arrowDeg, flag,
  } = arrowProps;

  const {
    arrow: arrowClass,
    tail: tailClass,
    head: headClass,
  } = customClasses;

  return (
    <svg
      viewBox={`0 0 ${w} ${w}`}
      className={classnames(
        classes['arc-arrow'],
        arrowClass,
      )}
    >
      <path
        className={classnames(
          classes.tail,
          tailClass,
        )}
        d={`M${startX} ${startY} A50 50 ${flag} ${endX} ${endY}`}
        stroke={color}
      />
      <path
        className={classnames(
          headClass,
        )}
        d={`M${-arrow / 2} ${-arrow * 0.7} L${arrow / 2} ${-arrow * 0.7} L${0} ${arrow / 2} Z`}
        transform={`translate(${arrowPos.x}, ${arrowPos.y}) rotate(${arrowDeg})`}
        fill={color}
      />
    </svg>
  );
};

ArcArrow.defaultProps = {
  start: 0,
  length: 0,
  reversed: false,
  color: 'black',
  classes: {},
};

ArcArrow.propTypes = {
  /**
   * What degree does the number start at
   */
  start: PropTypes.number,
  /**
   * How many degrees does the arc spawn
   */
  length: PropTypes.number,
  /**
   * True means the arrow is placed at the start of the arc
   * False means the arrow is placed at the end of the arc
   */
  reversed: PropTypes.bool,
  /**
   * Color for the arrow
   */
  color: PropTypes.string,
  /**
   * Custom CSS classes to apply to various parts of the arrow. The `arrow`
   * class gets applied to the base SVG, the `tail` class gets applied to the
   * tail of the arrow, and the `head` class gets applied to the arrow head.
   */
  classes: PropTypes.shape({
    arrow: PropTypes.string,
    tail: PropTypes.string,
    head: PropTypes.string,
  }),
};

export default ArcArrow;
