import PropTypes from 'prop-types';
import classnames from 'classnames';
import classes from './styles.module.scss';
import Block from './Block';
import legalStuffDescription from './content/legal-stuff';

/**
 * A standard footer to include at the bottom of a custom POLITICO interactive
 * story. This is generally used to include some legal disclaimers, a copyright
 * notice, and whatever additional credits or disclaimers are necessary for the
 * project. The "Legal Stuff" block is included by default, as is the copyright
 * notice, though you can disable the legal block and customize the copyright
 * notice with props.
 */
const InteractiveFooter = (props) => {
  const {
    content,
    copyrightYear,
    customCopyright,
    disableLegalBlock,
    classes: customClasses,
  } = props;

  const noContent = content.length === 0;

  const {
    container: containerClass,
    block: blockClass,
    copyright: copyrightClass,
  } = customClasses;

  return (
    <footer
      className={classnames(
        classes['interactive-footer'],
        containerClass,
      )}
    >

      {content.map((a) => (
        <Block
          key={a.title}
          title={a.title}
          description={a.description}
          fullWidth={a.fullWidth}
          className={blockClass}
          content={a.children}
        />
      ))}

      {!disableLegalBlock && (
        <Block
          title="Legal stuff"
          description={legalStuffDescription}
          fullWidth={noContent}
          className={blockClass}
        />
      )}

      {customCopyright || (
        <p
          className={classnames(
            classes.copyright,
            copyrightClass,
          )}
        >
          {`© ${copyrightYear} POLITICO LLC`}
        </p>
      )}
    </footer>
  );
};

InteractiveFooter.defaultProps = {
  content: [],
  classes: {},
  copyrightYear: 2023,
  customCopyright: undefined,
  disableLegalBlock: false,
};

InteractiveFooter.propTypes = {
  /**
   * The blocks of content to include as sections in the footer. The standard
   * block contains a title and a Markdown-formatted description. The title
   * of each block is used as its key, so all titles must be unique. An
   * individual block can be flagged as `fullWidth` if it should span both
   * footer columns on desktop. Finally, a block can specify `children`, which
   * is used as the block's content instead of the basic title/description.
   * Specify `children` if you want to take full custom control of the block's
   * content. Note that even if you specify `children`, the block must still
   * have a unique `title` for use as a key.
   */
  content: PropTypes.arrayOf(PropTypes.shape({
    title: PropTypes.string.isRequired,
    description: PropTypes.string,
    fullWidth: PropTypes.bool,
    children: PropTypes.node,
  })),
  /**
   * The year to use in the copyright at the bottom of the footer. This should
   * match the last year the project was published or updated.
   */
  copyrightYear: PropTypes.node,
  /**
   * The `customCopyright` replaces the default copyright notice entirely with
   * a custom block of content.
   */
  customCopyright: PropTypes.node,
  /**
   * Prevent the standard legal block from being included in the footer. Set
   * this to `true` if you want to include the legal stuff in your own content
   * directly.
   */
  disableLegalBlock: PropTypes.bool,
  /**
   * A set of custom classes to customize the styling of the footer. The
   * `container` class gets applied to the containing `<footer>` tag, the
   * `block` class gets applied to each individual block, and the `copyright`
   * class gets applied to the container of the default copyright notice.
   */
  classes: PropTypes.shape({
    container: PropTypes.string,
    block: PropTypes.string,
    copyright: PropTypes.string,
  }),
};

export default InteractiveFooter;
